package com.hemmis.sudoku;

import java.util.Arrays;
import java.util.EnumSet;

/**
 * Represents a cell in a sudoku field
 * @author sst
 *
 */
public class Cell {
	private EnumSet<Value> possibleValues;
	private Value value;
	private int row;
	private int column;
	
	public Cell(int row, int column, Value value) {
		this.row = row;
		this.column = column;
		this.value = value;
		possibleValues = EnumSet.of(value);
	}
	public Cell(int row, int column) {
		this.row = row;
		this.column = column;
		value = null;
		possibleValues = EnumSet.allOf(Value.class);
	}
	
	public boolean isKnown() {
		return null!=value;
	}
	
	/**
	 * Remove a possible value
	 * @param value the value to remove as possibility
	 * @throws NoMoreValuesException
	 */
	public void removePossibleValue(Value value) throws NoMoreValuesException {
		possibleValues.remove(value);
		evaluateKnownValue();
	}
	
	/**
	 * Evaluates the possible values and sets the value if only 1 possibility remains
	 * @throws NoMoreValuesException if the are no possible values left
	 */
	private void evaluateKnownValue() throws NoMoreValuesException {
		if(possibleValues.size() == 1) {
			value = possibleValues.iterator().next();
		} else if (possibleValues.size() == 0) {
			throw new NoMoreValuesException(this);
		}
	}
	
	@Override
	public String toString() {
		if (value ==null) {
			return Arrays.toString(possibleValues.toArray());
		} else {
			return value.toString();
		}
		
	}
	
	public void setValue(Value value) {
		this.value = value;
		possibleValues = EnumSet.of(value);
	}
	
	public Value getValue() {
		return value;
	}
	
	public long getNrOfPossibilities() {
		return possibleValues.size();
	}
	
	public int getRow() {
		return row;
	}
	
	public int getColumn() {
		return column;
	}
	
	public EnumSet<Value> getPossibleValues() {
		return possibleValues;
	}
	
	public boolean arePossiblitiesEqual(Cell otherCell) {
		if (getNrOfPossibilities()!= otherCell.getNrOfPossibilities()) {
			return false;
		}
		return possibleValues.equals(otherCell.getPossibleValues());
	}
 }
